const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'CertPoolCtrl'

module.exports = class CertPoolCtrl extends BaseOpenAPIClient {
  /**
   *
   * @description Returns information about certificate from server's certificates pool.
   * @param {Number} nVServerId Virtual server id (-1 for current, 0 for main server)
   * @param {Number} nFunction Certificate function (see "KLCERTP::CertificateFunction enum values")
   * @param {*} connection
   * @returns {Promise}
   */
  GetCertificateInfo ({nVServerId, nFunction}, connection) {
    return this.baseRequest({
      command: `${className}.GetCertificateInfo`,
      data: {nVServerId, nFunction},
      connection
    })
      .then(this.getResult)
  }

  /**
   *
   * @description Sets certificate of a given function for a given virtual server.
   * @param {Number} nVServerId Virtual server id (-1 for current, 0 for main server)
   * @param {Number} nFunction Certificate function (see "KLCERTP::CertificateFunction enum values")
   * @param {Object} pCertData Params with certificate data (see Common format for certificate params). If pCertData is empty or NULL, then certificate will be deleted.
   * @param {*} connection
   * @returns {Promise}
   */
  SetCertificate ({nVServerId, nFunction, pCertData}, connection) {
    return this.baseRequest({
      command: `${className}.SetCertificate`,
      data: {
        nVServerId, nFunction, pCertData
      },
      connection
    })
      .then(this.getResult)
  }
}
