const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'EventNotificationProperties'

module.exports = class EventNotificationProperties extends BaseOpenAPIClient {
  /**
   * Reads the default the notification settings, such as SMTP server properties, etc.
   * @param connection
   * @returns {Promise.<TResult>} - object containing the current notification settings
   * @constructor
   */
  GetDefaultSettings (args, connection) {
    return this.baseRequest({ command: `${className}.GetDefaultSettings`, connection })
      .then(this.getResult)
  }

  /**
   * Reads the notification limits
   * @param connection
   * @returns {Promise.<TResult>} - object containing the current notification limits
   * @constructor
   */
  GetNotificationLimits (args, connection) {
    return this.baseRequest({ command: `${className}.GetNotificationLimits`, connection })
      .then(this.getResult)
  }

  /**
   * Allows to setup the default the notification settings, such as SMTP server properties, etc.
   * @param pSettings - object containing the notification settings to be set
   * @param connection
   * @returns {Promise.<TResult>}
   * @constructor
   */
  SetDefaultSettings ({ pSettings }, connection) {
    return this.baseRequest({ command: `${className}.SetDefaultSettings`, data: { pSettings: this.serialize(pSettings) }, connection })
      .then(this.getResult)
  }

  /**
   * Allows to setup notification limits
   * @param connection
   * @returns {Promise.<TResult>} - object containing the notification limits to be set
   * @constructor
   */
  SetNotificationLimits ({ pSettings }, connection) {
    return this.baseRequest({ command: `${className}.SetNotificationLimits`, data: { pSettings: this.serialize(pSettings) }, connection })
      .then(this.getResult)
  }

  /**
   * Allows to test the notification settings, such as SMTP server properties, etc. by sending a test notification using
   * the provided notification settings
   * @param {int} eType - type of the notification to be tested
   * @param {object} pSettings - object containing the notification settings to be tested
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  TestNotification ({ eType, pSettings }, connection) {
    return this.baseRequest({ command: `${className}.TestNotification`, data: { eType, pSettings: this.serialize(pSettings) }, connection })
      .then(this.getResult)
  }
}
