const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'GroupSync'

/**
 * Access to group synchronization objects
 */
module.exports = class GroupSync extends BaseOpenAPIClient {
  /**
   * Acquire group synchronization properties.
   * Returns requested properties of the specified group synchronization
   * @param {int} nSync - id of the group synchronization
   * @param {Array<string>} arrFieldsToReturn - array of attribute names to return
   * @returns Promise<object> - container with values of required attributes
   */
  GetSyncInfo ({ nSync = 0, arrFieldsToReturn = [] }, connection) {
    return this.baseRequest({
      command: `${className}.GetSyncInfo`,
      data: this.serialize({
        nSync,
        arrFieldsToReturn
      }),
      connection
    })
      .then(this.getResult)
  }

  /**
   * Acquire group synchronization delivery time for the specified host.
   * Returns UTC time when the specified synchronization has been delivered to the specified host
   * @param {number} nSync - id of the group synchronization
   * @param {string} szwHostId - host name (see KLHST_WKS_HOSTNAME)
   * @param {*} connection
   */
  async GetSyncDeliveryTime ({ nSync, szwHostId }, connection) {
    const result = await this.baseRequest({
      command: `${className}.GetSyncDeliveryTime`,
      data: {
        nSync,
        szwHostId
      },
      connection
    })
    return this.getResult(result)
  }
}
