const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'GroupTaskControlApi'

module.exports = class GroupTaskControlApi extends BaseOpenAPIClient {

  /**
   * Returns all task data for a group/set task with a given object identity and revision.
   * @param {int} nObjId Task identity
   * @param {int} nRevision Task revision id
   * @param {any} connection 
   * @returns list of attributes of executable file
   */
    async GetTaskByRevision ({nObjId, nRevision}, connection) {
        const body = await this.baseRequest({
            command: `${className}.GetTaskByRevision`,
            data: {
                nObjId,
                nRevision
            },
            connection
        })
        return this.getBody(body)
    }

   /**
   * Request statistics of the given tasks.
   * @param {arr<int>} pTasksIds Task identity
   * @param {any} connection 
   * @returns list of attributes of executable file
   */
  async RequestStatistics ({pTasksIds}, connection) {
        const body = await this.baseRequest({
            command: `${className}.RequestStatistics`,
            data: {
                pTasksIds
            },
            connection
        })
        return this.getBody(body)
    }

   /**
   * Restore task from revision.
   * Rolls back the group/set task specified by nObjId to the revision nRevision.
   * @param {int} nObjId Task identity
   * @param {int} nRevision Task revision id
   * @param {any} connection 
   * @returns list of attributes of executable file
   */
  async RestoreTaskFromRevision ({nObjId, nRevision}, connection) {
    const body = await this.baseRequest({
        command: `${className}.RestoreTaskFromRevision`,
        data: {
            nObjId,
            nRevision
        },
        connection
    })
    return this.getBody(body)
}

}
