const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'HostGroup'

module.exports = class HostGroup extends BaseOpenAPIClient {
  /**
   * Получение идентификатора предопределённой группы "Managed computers"
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GroupIdGroups (noParams, connection) {
    return this.baseRequest({
      command: `${className}.GroupIdGroups`,
      data: {},
      connection
    })
      .then(this.getResult)
  }

  /**
   * Получение идентификатора предопределённой группы "Unassigned computers"
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GroupIdUnassigned (noParams, connection) {
    return this.baseRequest({
      command: `${className}.GroupIdUnassigned`,
      data: {},
      connection
    })
      .then(this.getResult)
  }

  /**
   * Get id of predefined group "Master server".
   * @param {Connection} connection
   */
  async GroupIdSuper (noParams, connection) {
    const result = await this.baseRequest({
      command: `${className}.GroupIdSuper`,
      data: {},
      connection
    })
    const groupIdSuper = this.getResult(result)
    return groupIdSuper
  }

  async GetGroupInfo ({ nGroupId, deserialize = false }, connection) {
    const result = await this.baseRequest({
      command: `${className}.GetGroupInfo`,
      data: { nGroupId },
      connection
    })
    const groupInfo = deserialize ? this.deserialize(result) : this.getResult(result)
    return groupInfo
  }

  /**
   * Поиск хостов с помощью фильтров
   * @param {string} wstrFilter - фильтр, содеражищий условия по атрибутам хоста
   * @param {string[]} vecFieldsToReturn - список названий атрибутов хоста, которые должны быть возвращены
   * @param {Object[]} vecFieldsToOrder - список контейнеров (объектов) с настройками сортировки
   * @param {} pParams - дополнительные праметры
   * @param {number} lMaxLifeTime - максимальное время жизни итератора в секундах, не более чем 7200
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindHosts ({ wstrFilter, vecFieldsToReturn, vecFieldsToOrder = [], pParams = null, lMaxLifeTime = 7200, skipDeserialization = false } = {}, connection) {
    return this.baseRequest({
      command: `${className}.FindHosts`,
      data: { wstrFilter, vecFieldsToReturn, vecFieldsToOrder, pParams, lMaxLifeTime },
      connection
    })
      .then(body => skipDeserialization ? body.body : this.deserialize(body, 'strAccessor'))
  }

  /**
   * Перемещение нескольких хостов в группу администрирования
   * @param {number} nGroup - идентификатор группы назначения
   * @param {Array.String} pHostNames - список имён хостов (KLHST_WKS_HOSTNAME)
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  MoveHostsToGroup ({ nGroup, pHostNames } = {}, connection) {
    return this.baseRequest({
      command: `${className}.MoveHostsToGroup`,
      data: { nGroup, pHostNames },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Удаление нескольких хостов
   * @param {Array.String} pHostNames - имена удаляемых хостов (KLHST_WKS_HOSTNAME)
   * @param {boolean} bForceDestroy - true: хост будет удалён; false: управляемое устройство будет перемещено в Unassigned
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  RemoveHosts ({ pHostNames, bForceDestroy = false } = {}, connection) {
    return this.baseRequest({
      command: `${className}.RemoveHosts`,
      data: { pHostNames, bForceDestroy },
      connection
    })
      .then(this.getResult)
  }

  GetSubgroups (params, connection) {
    return this.baseRequest({
      command: `${className}.GetSubgroups`,
      data: params,
      connection
    })
      .then(this.deserialize)
  }
  AddGroup (params, connection) {
    return this.baseRequest({
      command: `${className}.AddGroup`,
      data: params,
      connection
    })
      .then(this.deserialize)
  }
  UpdateGroup ({ nGroup, pInfo }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateGroup`,
      data: {
        nGroup,
        pInfo: this.serialize(pInfo)
      },
      connection
    })
      .then(this.deserialize)
  }
  RemoveGroup ({ nGroup, nFlags = 1 }, connection) {
    return this.baseRequest({
      command: `${className}.RemoveGroup`,
      data: { nGroup, nFlags },
      connection
    })
      .then(result => this.getResult(result, 'strActionGuid'))
  }

  /**
   * Finds groups that satisfy conditions from filter string wstrFilter, and creates a server-side collection of found groups
   * @param {string} wstrFilter - filter string, contains a condition over List of group attributes
   * @param {Array} vecFieldsToReturn - array of group attribute names to return. See List of group attributes for attribute names
   * @param {Array} vecFieldsToOrder -  array of containers each of them containing two attributes
   * @param {Object} pParams - extra options
   * @param {number} lMaxLifeTime - max result-set lifetime in seconds, not more than 7200
   * @param {Connection} connection
   */
  async FindGroups ({ wstrFilter, vecFieldsToReturn = [], vecFieldsToOrder = [], pParams = null, lMaxLifeTime = 7200 }, connection) {
    const body = await this.baseRequest({
      command: `${className}.FindGroups`,
      data: {
        wstrFilter,
        vecFieldsToReturn,
        vecFieldsToOrder,
        pParams,
        lMaxLifeTime
      },
      connection
    })
    const result = this.getResult(body, 'strAccessor')
    return result
  }

  GetStaticInfo (params, connection) {
    // pValues
    return this.baseRequest({
      command: `${className}.GetStaticInfo`,
      data: {
        pValues: params
      },
      connection
    }).then(this.getResult)
  }

  /**
   * Получение информации о хосте
   * @param {string} strHostName - параметр хоста KLHST_WKS_HOSTNAME
   * @param {Array.string} pFields2Return - список полей, которые должны быть возвращены
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetHostInfo ({strHostName, pFields2Return, deserialize = false}, connection) {
    return this.baseRequest({
      command: `${className}.GetHostInfo`,
      data: {
        strHostName,
        pFields2Return
      },
      connection
    }).then(deserialize ? this.deserialize : this.getResult)
  }

  /**
   * Обновление (сохранение) информации о хосте
   * @param {string} strHostName - имя хоста (KLHST_WKS_HOSTNAME)
   * @param {object} pInfo - контейнер с атрибутами хоста, которые должны быть обновлены
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateHost ({ strHostName, pInfo }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateHost`,
      data: {
        strHostName,
        pInfo
      },
      connection
    }).then(this.getBody)
  }
  // eslint-disable-next-line camelcase
  SS_GetNames ({strHostName, strType, strProduct, strVersion}, connection) {
    return this.baseRequest({
      command: `${className}.SS_GetNames`,
      data: {
        strHostName, strType, strProduct, strVersion
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Чтение из хранилища настроек (settings storage) устройства
   * @param {string} strHostName - имя устройства
   * @param {string} strType - тип хранилища
   * @param {string} strProduct - идентификатор продукта
   * @param {string} strVersion - версия продукта
   * @param {string} strSection - секция хранилища
   * @param {boolean} skipDeserialization - флаг, нужно ли десериализовывать ответ
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  // eslint-disable-next-line camelcase
  SS_Read ({strHostName, strType = 'SS_SETTINGS', strProduct, strVersion, strSection, skipDeserialization = false}, connection) {
    return this.baseRequest({
      command: `${className}.SS_Read`,
      data: {
        strHostName, strType, strProduct, strVersion, strSection
      },
      connection
    }).then(skipDeserialization ? this.getBody : this.deserialize)
  }
  // eslint-disable-next-line camelcase
  SS_Write ({strHostName, strType, strProduct, strVersion, strSection, nOption, pSettings, skipSerialization}, connection) {
    let _pSettings = skipSerialization ? pSettings : this.serialize(pSettings)
    return this.baseRequest({
      command: `${className}.SS_Write`,
      data: {
        strHostName, strType, strProduct, strVersion, strSection, nOption, pSettings: _pSettings
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Delete section from host settings storage.
   * @param strHostName - host name (unique server-generated string)
   * @param strType - type of storage (for example: "SS_SETTINGS")
   * @param strProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param strVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param strSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param connection
   * @returns {Promise}
   * @constructor
   */
  // eslint-disable-next-line camelcase
  async SS_DeleteSection ({strHostName, strType = 'SS_SETTINGS', strProduct, strVersion, strSection}, connection) {
    return this.baseRequest({
      command: `${className}.SS_DeleteSection`,
      data: {
        strHostName, strType, strProduct, strVersion, strSection
      },
      connection
    })
  }

  /**
   * Create section in host settings storage
   * @param strHostName - host name (unique server-generated string)
   * @param strType - type of storage (for example: "SS_SETTINGS")
   * @param strProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param strVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param strSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @returns {Promise}
   * @constructor
   */
  // eslint-disable-next-line camelcase
  async SS_CreateSection ({strHostName, strType = 'SS_SETTINGS', strProduct, strVersion, strSection}, connection) {
    return this.baseRequest({
      command: `${className}.SS_CreateSection`,
      data: {
        strHostName, strType, strProduct, strVersion, strSection
      },
      connection
    })
  }

  /**
   * Получение информации об установленных на хосте продуктах
   * @param {string} strHostName - идентификатор устройства aka KLHST_WKS_HOSTNAME
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetHostProducts ({strHostName}, connection) {
    return this.baseRequest({
      command: `${className}.GetHostProducts`,
      data: {
        strHostName
      },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Поиск пользователей, которые удовлетворяют условиям фильтра (параметр strFilter)
   * @param {string} strFilter - фильтр
   * @param {string[]} pFieldsToReturn - список атрибутов пользователя, которые должны быть возвращены
   * @param {Object[]} pFieldsToOrder - массив контейнеров (объектов), содепржащих настройки сортировки
   * @param {number} lMaxLifeTime - максимальное время жизни итератора в секундах, не более чем 7200
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindUsers ({ strFilter, pFieldsToReturn, pFieldsToOrder = [], lMaxLifeTime = 7200 } = {}, connection) {
    return this.baseRequest({
      command: `${className}.FindUsers`,
      data: { strFilter, pFieldsToReturn, pFieldsToOrder, lMaxLifeTime },
      connection
    })
      .then(body => this.getResult(body, 'strAccessor'))
  }

  /**
   * Получение серверного идентификатора для управления задачми устройства
   * @param {string} strHostName - имя хоста (KLHST_WKS_HOSTNAME)
   * @param {Connection} connection
   * @return {Promise.<{result}>}
   * @constructor
   */
  GetHostTasks ({strHostName}, connection) {
    return this.baseRequest({
      command: `${className}.GetHostTasks`,
      data: {strHostName},
      connection
    })
      .then(this.getResult)
  }

  /**
   * Обнуление количества найденных вирусов на хостах
   * @param {Array.string} pHostNames - список имён хостов, у которых нужно обнулить количество найденных вирусов
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  ZeroVirusCountForHosts ({ pHostNames }, connection) {
    return this.baseRequest({
      command: `${className}.ZeroVirusCountForHosts`,
      data: {pHostNames},
      connection
    })
      .then(this.getBody)
  }

  /**
   * Получение списка установленных для продукта патчей
   * @param {string} strHostName - имя устройства (KLHST_WKS_HOSTNAME)
   * @param {string} strProductName - идентификатор продукта
   * @param {string} strProductVersion - версия продукта
   * @param {Connection} connection
   * @return {Promise.<{result}|*>}
   */
  async GetHostfixesForProductOnHost ({strHostName, strProductName, strProductVersion}, connection) {
    const installedPatchesResponse = await this.baseRequest({
      command: `${className}.GetHostfixesForProductOnHost`,
      data: {strHostName, strProductName, strProductVersion},
      connection
    })
    return this.getResult(installedPatchesResponse)
  }

  RestartNetworkScanning ({nType}, connection) {
    return this.baseRequest({
      command: `${className}.RestartNetworkScanning`,
      data: {nType},
      connection
    }).then(this.getResult)
  }

  /**
   * Acquire administration group id by its name and id of parent group.
   * @param nParent - Id of parent group
   * @param strName - name of group
   * @returns {Promise<any>}
   */
  GetGroupId ({ nParent, strName }, connection) {
    return this.baseRequest({
      command: `${className}.GetGroupId`,
      data: { nParent, strName },
      connection
    }).then(this.getResult)
  }

  /**
   * List of Windows domain in the network.
   * @returns {Promise<object>}
   */
  GetDomains (noParams, connection) {
    return this.baseRequest({
      command: `${className}.GetDomains`,
      connection
    }).then(this.deserialize)
  }

  /**
   * Return a list of workstation names in the domain
   * The information is obtained from the domain controller. This call returns the full list of workstations in the domain, even if the workstation is now turned off
   * @param domain - domain name
   * @returns {Promise<any>} - array of hosts in domain
   * @constructor
   */
  GetDomainHosts ({ domain }, connection) {
    return this.baseRequest({
      command: `${className}.GetDomainHosts`,
      data: { domain },
      connection
    }).then(this.deserialize)
  }

  /**
   * Finds hosts asynchronously that satisfy conditions from filter string wstrFilter, and creates a server-side collection of found hosts
   * @param wstrFilter - filter string, contains a condition over host attributes
   * @param vecFieldsToReturn - array of host attribute names to return
   * @param vecFieldsToOrder - array of containers each of them containing two attributes "Name" and "Asc"
   * @param pParams - extra options
   * @param lMaxLifeTime - max result-set lifetime in seconds, not more than 7200
   * @returns {Promise<string>} - strRequestId
   */
  FindHostsAsync ({ wstrFilter, vecFieldsToReturn, vecFieldsToOrder = [], pParams, lMaxLifeTime = 7200 }, connection) {
    return this.baseRequest({
      command: `${className}.FindHostsAsync`,
      data: this.serialize({
        wstrFilter,
        vecFieldsToReturn,
        vecFieldsToOrder,
        pParams,
        lMaxLifeTime
      }),
      connection
    }).then(this.getBody)
  }

  /**
   * Get result of FindHostsAsync operation
   * @param strRequestId - identity of asynchronous operation
   * @returns {Promise<object>} { strAccessor, pFailedSlavesInfo } - result-set ID, identifier of the server-side ordered collection of found hosts. The result-set is destroyed and associated memory is freed in following cases; information about slave servers the search for which failed due to various reasons, contains array KLGRP_FAILED_SLAVES_PARAMS
   */
  FindHostsAsyncGetAccessor ({ strRequestId }, connection) {
    return this.baseRequest({
      command: `${className}.FindHostsAsyncGetAccessor`,
      data: {
        strRequestId
      },
      connection
    }).then(this.getBody)
  }

  GetInstanceStatistics ({vecFilterFields}, connection) {
    return this.baseRequest({
      command: `${className}.GetInstanceStatistics`,
      data: {
        vecFilterFields
      },
      connection
    }).then(body => this.getResult(body, 'PxgRetVal'))
  }

  /**
   * Add hosts to synchronization
   * @param {Array.string} pHostNames - array of host names
   * @param {string} strSSType - setting storage identity (empty string means synchronization of all setting storages)
   * @param {Connection} connection
   * @return {Promise.string>}
   */
  async AddHostsForSync ({ pHostNames, strSSType = '' }, connection) {
    const result = await this.baseRequest({
      command: `${className}.AddHostsForSync`,
      data: {
        pHostNames,
        strSSType
      },
      connection
    })
    return this.getResult(result, 'strActionGuid')
  }

  /**
   * Return array of product components for specified host and product.
   * @param {string} strHostName - host name
   * @param {string} strProductName - product name
   * @param {string} strProductVersion - product version
   * @param {Connection} connection
   * @return {Promise.<{result}>}
   * @constructor
   */
  GetComponentsForProductOnHost ({strHostName, strProductName, strProductVersion}, connection) {
    return this.baseRequest({
      command: `${className}.GetComponentsForProductOnHost`,
      data: {strHostName, strProductName, strProductVersion},
      connection
    }).then(this.deserialize)
  }

  /**
   * Update attributes of multiple computers.
   * @param {array} pArrHostIds - Array of up to 100 strings. Each entry is a host name, a unique server-generated string (see KLHST_WKS_HOSTNAME attribute). It is NOT the same as computer network name (DNS-, FQDN-, NetBIOS-name)
   * @param {params} pProperties - container with host attributes to be modified. See List of host attributes. Following properties can be specified: KLHST_MANAGED_OTHER_SERVER
   * @param {Connection} connection
   * @return {Promise.<{result}>} - Array of failed identifiers. If Administration Server failed to change host attributes it puts host identifier into this array.
   * @constructor
   */
  UpdateHostsMultiple ({pArrHostIds, pProperties}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateHostsMultiple`,
      data: {pArrHostIds, pProperties},
      connection
    }).then(this.deserialize)
  }

  /**
   * Find incident by filter string.
   * Finds incidents that satisfy conditions from filter string strFilter.
   * @param {string} strFilter - incident filtering expression (see Search filter syntax). See the list of incident attributes that can be used in this expression in Remarks section below
   * @param {array} pFieldsToReturn - array of incident attribute names to return. See List of incident attributes for attribute names
   * @param {array} pFieldsToOrder - array of containers each of them containing two attributes: -"Name" of type String, name of attribute used for ordering (see Remarks below); -"Asc" of type Boolean, ascending if true descending otherwise;
   * @param {number} lMaxLifeTime - max lifetime of accessor (sec)
   * @param {Connection} connection
   * @return {Promise.<{strAccessor}>} - result-set ID, identifier of the server-side ordered collection of found incidents. The result-set is destroyed and associated memory is freed in following cases:
   * -Passed lMaxLifeTime seconds after last access to the result-set (by methods ChunkAccessor.GetItemsCount and ChunkAccessor.GetItemsChunk
   * -Session to the Administration Server has been closed.
   * -ChunkAccessor.Release has been called.
   * @constructor
   */
  FindIncidents ({strFilter, pFieldsToReturn, pFieldsToOrder, lMaxLifeTime}, connection) {
    return this.baseRequest({
      command: `${className}.FindIncidents`,
      data: {strFilter, pFieldsToReturn, pFieldsToOrder, lMaxLifeTime},
      connection
    }).then(body => this.getResult(body, 'strAccessor'))
  }

  /**
   *Creates new incident.
   *
   * @param {params} pData - container with incident attributes Attributes can be set:
   * - "KLINCDT_SEVERITY" (required)
   * - "KLINCDT_ADDED" (required)
   * - "KLINCDT_BODY" (required)
   * - "KLINCDT_IS_HANDLED" (optional, false by default)
   * - "KLHST_WKS_HOSTNAME" (required)
   * @param {*} connection
   * @returns {Promise<int>} - unique incident id
   */
  AddIncident ({pData}, connection) {
    return this.baseRequest({
      command: `${className}.AddIncident`,
      data: {pData},
      connection
    }).then(this.deserialize)
  }

  /**
   * Deletes incident
   *Grou
   * @param {number} {nId} - incident id
   * @param {*} connection
   * @returns {Promise}
   */
  DeleteIncident ({nId}, connection) {
    return this.baseRequest({
      command: `${className}.DeleteIncident`,
      data: {nId},
      connection
    }).then(this.deserialize)
  }

  /**
   * Modifies properties of an existing incident.
   *
   * @param {number} {nId} - incident id
   * @param {params} pData - container with incident attributes Attributes can be set:
   * - "KLINCDT_SEVERITY" (required)
   * - "KLINCDT_ADDED" (required)
   * - "KLINCDT_BODY" (required)
   * - "KLINCDT_IS_HANDLED" (optional, false by default)
   * - "KLHST_WKS_HOSTNAME" (required)
   * @param {*} connection
   * @returns {Promise}
   */
  UpdateIncident ({nId, pData}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateIncident`,
      data: {nId, pData},
      connection
    }).then(this.deserialize)
  }
}
