const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'KsnInternal'

module.exports = class KsnInternal extends BaseOpenAPIClient {
  /**
   * Verify KPSN settings
   * @param {binary} pSettigns - KPSN settings file (in PKCS7 format)
   * @param connection
   * @returns {Promise.<TResult>} - Params with EULA text and localization
   * @constructor
   */
  VerifyNKsnSettings ({ pSettings }, connection) {
    return this.baseRequest({ command: `${className}.VerifyNKsnSettings`, data: { pSettings }, connection })
      .then(this.getBody)
  }

  /**
   * Get KPSN eula
   * @param {string} wstrNKsnLoc - Localization ('ru', 'en')
   * @param connection
   * @returns {Promise.<TResult>} - Params with EULA text and localization
   * @constructor
   */
  GetNKsnEula ({ wstrNKsnLoc }, connection) {
    return this.baseRequest({ command: `${className}.GetNKsnEula`, data: { wstrNKsnLoc }, connection })
      .then(this.getBody)
  }

  /**
   * Get all KPSN eula
   * @param connection
   * @returns {Promise.<TResult>} - Array of eula
   * @constructor
   */
  GetNKsnEulas (args, connection) {
    return this.baseRequest({ command: `${className}.GetNKsnEulas`, connection })
      .then(this.getBody)
  }

  /**
   * Check connection with KSN cloud (or KPSN)
   * @param connection
   * @returns {Promise.<TResult>} - Returns true if connection checked. Otherwise - false.
   * @constructor
   */
  CheckKsnConnection (args, connection) {
    return this.baseRequest({ command: `${className}.CheckKsnConnection`, connection })
      .then(this.getResult)
  }
}
