const BaseOpenAPIClient = require('../base-open-api-client')

module.exports = class LicenseInfoSync extends BaseOpenAPIClient {
  /** Acquire serial numbers of the active and reserved licenses on the host. */
  async AcquireKeysForProductOnHost ({ szwHostName, szwProduct, szwVersion }, connection) {
    return this.getBody(await this.baseRequest({
      command: `${this.constructor.name}.AcquireKeysForProductOnHost`,
      data: { szwHostName, szwProduct, szwVersion },
      connection
    }))
  }

  /**
  * Get host-specific key data.
  * @param {string} szwHostName Host ID
  * @param {string} szwProduct Product ID
  * @param {string} szwVersion Version
  * @param {string} szvSerial Key serial number
  * @param {*} connection connection
  */
  async GetKeyDataForHost ({ szwHostName, szwProduct, szwVersion, szwSerial }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.GetKeyDataForHost`,
      data: { szwHostName, szwProduct, szwVersion, szwSerial },
      connection
    }))
  }

  /**
   * Check if license is suitable for being used by the adm. server
   * @param pKeyInfo - License attribute container
   * @param bAsCurrent - true to check it as an active one, false to check it as a reserved one
   * @returns {Promise} - Throws exception in case of error
   */
  IsLicForSaasValid2 ({ pKeyInfo, bAsCurrent }, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.IsLicForSaasValid2`,
      data: {
        pKeyInfo,
        bAsCurrent
      },
      connection
    })
  }

  /**
   * Check whether the key's product id belongs to the Public Cloud product ids list
   * @param nProductId - Product ID, mandatory
   * @returns {Promise} - true if product id belongs to the Public Cloud product ids list, false otherwise
   */
  IsPCloudKey ({ nProductId }, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.IsPCloudKey`,
      data: {
        nProductId
      },
      connection
    }).then(this.getResult)
  }

  /**
   * Force synchronization of subscription licenses' metadata
   * @returns {Promise} - Request ID used to subscribe to the event that is triggered when operation is complete
   */
  SynchronizeLicInfo2 (noParams, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.SynchronizeLicInfo2`,
      data: {},
      connection
    }).then(this.getResult)
  }

  /**
   * Install adm. server's license
   * @param pKeyInfo - License attribute container
   * @param bAsCurrent - true to install it as an active one, false to install it as a reserved one
   */
  TryToInstallLicForSaas2 ({ pKeyInfo, bAsCurrent }, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.TryToInstallLicForSaas2`,
      data: {
        pKeyInfo,
        bAsCurrent
      },
      connection
    })
  }

  /**
   * Uninstall adm. server's license
   * @param bCurrent - true to install active license, otherwise uninstall the reserved one
   * @returns {Promise}
   */
  TryToUnistallLicense ({ bCurrent }, connection) {
    return this.baseRequest({
      command: `${this.constructor.name}.TryToUnistallLicense`,
      data: {
        bCurrent
      },
      connection
    })
  }
}
