const GroupSyncIterator = require('../group-sync-iterator')

module.exports = class LicenseKeys extends GroupSyncIterator {
  /** Get an array of hosts that are currently using the specified key. */
  async AcquireKeyHosts ({ pInData, pFields, pFieldsToOrder, pOptions, lTimeoutSec }, connection) {
    return this.deserialize(await this.getAllItems({
      command: `${this.constructor.name}.AcquireKeyHosts`,
      data: { pInData, pFields, pFieldsToOrder, pOptions, lTimeoutSec },
      connection
    }))
  }

  /** Adjust adm. server's license attributes. */
  async AdjustKey ({ pData }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.AdjustKey`,
      data: { pData },
      connection
    }))
  }

  /** Check if license can be installed to the adm. server.
   *  License is treated as a valid one if it is suitable for being used by the adm. server itself (its AppId is 1017, it is not expired etc.) */
  async CheckIfSaasLicenseIsValid ({ pInData, bAsCurrent }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.CheckIfSaasLicenseIsValid`,
      data: { pInData, bAsCurrent },
      connection
    }))
  }

  /** Enumerate keys. */
  async EnumKeys ({ pFields, pFieldsToOrder, pOptions, lTimeoutSec }, connection) {
    return this.deserialize(await this.getAllItems({
      command: `${this.constructor.name}.EnumKeys`,
      data: { pFields, pFieldsToOrder, pOptions, lTimeoutSec },
      connection
    }))
  }

  /** Get data of a key. */
  async GetKeyData ({ pKeyInfo }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.GetKeyData`,
      data: { pKeyInfo },
      connection
    }))
  }

  /** Install a key on the server. */
  async InstallKey ({ pKeyInfo }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.InstallKey`,
      data: { pKeyInfo },
      connection
    }))
  }

  /** Install an adm. server's license. */
  async SaasTryToInstall ({ pInData, bAsCurrent }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.SaasTryToInstall`,
      data: { pInData, bAsCurrent },
      connection
    }))
  }

  /** Uninstall an adm. server's license. */
  async SaasTryToUninstall ({ bCurrent }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.SaasTryToUninstall`,
      data: { bCurrent },
      connection
    }))
  }

  /** Uninstall a key from the adm. server. */
  async UninstallKey ({ pKeyInfo }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.UninstallKey`,
      data: { pKeyInfo },
      connection
    }))
  }

  /**
   * @description Download license key files from activation key servers V1.
   * @param {string} { wstrActivationCode } activation code in format XXXXX-XXXXX-XXXXX-XXXXX, mandatory.
   * @param {*} connection
   * @returns {Array} array of license key files related to the specified code.
   */
  async DownloadKeyFiles ({ wstrActivationCode }, connection) {
    return this.deserialize(await this.baseRequest({
      command: `${this.constructor.name}.DownloadKeyFiles`,
      data: { wstrActivationCode },
      connection
    }))
  }
}
