const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'PackagesApi'

module.exports = class PackagesApi extends BaseOpenAPIClient {
  GetUserAgreements (data, connection) {
    return this.baseRequest({
      command: `${className}.GetUserAgreements`,
      data: {},
      connection
    })
      .then(this.getResult)
  }
  GetEulaText ({nEulaId}, connection) {
    return this.baseRequest({
      command: `${className}.GetEulaText`,
      data: {nEulaId},
      connection
    })
      .then(this.getBody)
  }
  AcceptEulas ({vecEulaIDs = []}, connection) {
    return this.baseRequest({
      command: `${className}.AcceptEulas`,
      data: {vecEulaIDs},
      connection
    })
      .then(this.deserialize)
  }
  GetPackages2 (data, connection) {
    return this.baseRequest({
      command: `${className}.GetPackages2`,
      data: {},
      connection
    })
      .then(this.deserialize)
  }

  GetPackages (data, connection) {
    return this.baseRequest({
      command: `${className}.GetPackages`,
      data: {},
      connection
    })
      .then(this.deserialize)
  }

  DeleteExecutablePkg ({nExecId}, connection) {
    return this.baseRequest({
      command: `${className}.DeleteExecutablePkg`,
      data: {
        nExecId: +nExecId
      },
      connection
    })
      .then(this.deserialize)
  }

  RemovePackage ({nPackageId}, connection) {
    return this.baseRequest({
      command: `${className}.RemovePackage`,
      data: {
        nPackageId: +nPackageId
      },
      connection
    })
      .then(this.deserialize)
  }

  GetPackageInfo2 ({nPackageId, skipDeserialize}, connection) {
    return this.baseRequest({
      command: `${className}.GetPackageInfo2`,
      data: {
        nPackageId
      },
      connection
    })
      .then(skipDeserialize ? this.getResult : this.deserialize)
  }

  GetPackageInfo ({nPackageId}, connection) {
    return this.baseRequest({
      command: `${className}.GetPackageInfo`,
      data: {
        nPackageId
      },
      connection
    })
      .then(this.deserialize)
  }

  SS_GetNames ({
    wstrStorageDescr,
    wstrName,
    wstrVersion,
    nTimeoutMsec = 0
  }, connection) {
    return this.baseRequest({
      command: `${className}.SS_GetNames`,
      data: {
        wstrStorageDescr,
        wstrName,
        wstrVersion,
        nTimeoutMsec
      },
      connection
    })
      .then(this.deserialize)
  }

  SS_Read ({
    wstrStorageDescr,
    wstrName,
    wstrVersion,
    nTimeoutMsec = 0,
    wstrSection,
    skipDeserialize
  }, connection) {
    return this.baseRequest({
      command: `${className}.SS_Read`,
      data: {
        wstrStorageDescr,
        wstrName,
        wstrVersion,
        nTimeoutMsec,
        wstrSection
      },
      connection
    })
      .then(skipDeserialize ? this.getResult : this.deserialize)
  }

  SS_Write ({
    wstrStorageDescr,
    wstrName,
    wstrVersion,
    nTimeoutMsec = 0,
    wstrSection,
    pData,
    nOperationType = 3,
    skipSerialize
  }, connection) {
    return this.baseRequest({
      command: `${className}.SS_Write`,
      data: {
        wstrStorageDescr,
        wstrName,
        wstrVersion,
        wstrSection,
        pData: skipSerialize ? pData : this.serialize(pData),
        nTimeoutMsec,
        nOperationType
      },
      connection
    })
      .then(this.getResult)
  }

  RenamePackage ({ nPackageId,
    wstrNewPackageName
  }, connection) {
    return this.baseRequest({
      command: `${className}.RenamePackage`,
      data: {
        nPackageId,
        wstrNewPackageName
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Возвращает список несовместимых с пакетом программ
   * @param nPackageId - id пакета установки
   * @param connection
   * @returns {Promise.<Arrays<object>} - список несовместимых программ
   * @constructor
   */
  async GetIncompatibleAppsInfo ({ nPackageId }, connection) {
    return this.baseRequest({
      command: `${className}.GetIncompatibleAppsInfo`,
      data: this.serialize({
        nPackageId
      }),
      connection
    })
      .then(this.getResult)
  }

  /**
   * Set incompatible apps info
   * @param nPackageId - Package ID
   * @param bRemoveIncompatibleApps - Remove incompatible apps flag
   * @param connection
   * @returns {Promise.<TResult>}
   * @constructor
   */
  async SetRemoveIncompatibleApps ({ nPackageId, bRemoveIncompatibleApps }, connection) {
    return this.baseRequest({
      command: `${className}.SetRemoveIncompatibleApps`,
      data: this.serialize({
        nPackageId,
        bRemoveIncompatibleApps
      }),
      connection
    })
      .then(this.getResult)
  }

  /**
     * Read package file content
     * @param {number} nPackageId - Package ID
     * @param {string} wstrFileName - File name
     * @param connection
     * @returns {Promise.<string>} pData - Binary representation of file contents
     * @constructor
     */
  async ReadPkgCfgFile ({ nPackageId, wstrFileName }, connection) {
    return this.baseRequest({
      command: `${className}.ReadPkgCfgFile`,
      data: this.serialize({
        nPackageId,
        wstrFileName
      }),
      connection
    })
      .then(this.getResult)
  }

  /**
     * Write package file content
     * @param {number} nPackageId - Package ID
     * @param {string} wstrFileName - File name
     * @param {string} pData - Binary representation of file contents
     * @param connection
     * @returns {Promise.<TResult>}
     * @constructor
     */
  async WritePkgCfgFile ({ nPackageId, wstrFileName, pData }, connection) {
    return this.baseRequest({
      command: `${className}.WritePkgCfgFile`,
      data: this.serialize({
        nPackageId,
        wstrFileName,
        pData: pData
      }),
      connection
    })
      .then(this.getResult)
  }

  /**
     * Update bases in packages (asynchronously)
     * @param {object} pParams - Input data container
     * @param {Array<int>} pParams.KLTSK_RI_PACKAGES_IDS - Packages IDs (paramArray <paramInt>, optional) If parameter not exist, all packages are subject to update
     * @constructor
     */
  async UpdateBasesInPackagesAsync ({pParams}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateBasesInPackagesAsync`,
      data: {
        pParams: this.serialize(pParams)
      },
      connection
    }).then(this.getResult)
  }

  /**
     * Cancel an asynchronous bases update operation
     * @param {string} wstrRequestId - Request ID
     * @constructor
     */
  async CancelUpdateBasesInPackages ({ wstrRequestId }, connection) {
    return this.baseRequest({
      command: `${className}.CancelUpdateBasesInPackages`,
      data: {
        wstrRequestId
      },
      connection
    }).then(this.getResult)
  }

  /**
     * Set license key.
     * @param {int} nPackageId  Package ID.
     * @param {wstring} wstrKeyFileName Key file name.
     * @param {binary} pData  Key data.
     * @param {boolean} bRemoveExisting  Remove all existing license keys.
     * @constructor
     */
  async SetLicenseKey ({ nPackageId, wstrKeyFileName, pData, bRemoveExisting }, connection) {
    return this.baseRequest({
      command: `${className}.SetLicenseKey`,
      data: {
        nPackageId, wstrKeyFileName, pData, bRemoveExisting
      },
      connection
    }).then(this.getBody)
  }

  /**
     * Get license key.
     * @param {int} nPackageId  Package ID.
     * @returns {string} wstrKeyFileName Key file name.
     * @returns {binary} pMemoryChunk Key data.
     */
  async GetLicenseKey ({ nPackageId }, connection) {
    return this.baseRequest({
      command: `${className}.GetLicenseKey`,
      data: {
        nPackageId
      },
      connection
    }).then(this.getBody)
  }
}
