const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'ReportManager'

module.exports = class ReportManager extends BaseOpenAPIClient {
  async EnumReports (data, connection) {
    const body = await this.baseRequest({
      command: `${className}.EnumReports`,
      connection
    })
    return this.deserialize(body)
  }

  async EnumReportTypes (data, connection) {
    const body = await this.baseRequest({
      command: `${className}.EnumReportTypes`,
      connection
    })
    return this.deserialize(body)
  }

  async GetReportTypeDetailedInfo (lReportTypeId, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetReportTypeDetailedInfo`,
      data: { lReportTypeId },
      connection
    })
    return this.deserialize(body)
  }

  async GetReportInfo (lReportId, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetReportInfo`,
      data: { lReportId },
      connection
    })
    return this.deserialize(body)
  }

  async AddReport (data, connection) {
    const body = await this.baseRequest({
      command: `${className}.AddReport`,
      data: { pReportInfo: this.serialize(data) },
      connection
    })
    return this.getResult(body)
  }

  async UpdateReport ({ lReportId, data }, connection) {
    const body = await this.baseRequest({
      command: `${className}.UpdateReport`,
      data: {
        lReportId,
        pReportInfo: this.serialize(data)
      },
      connection
    })
    return this.deserialize(body)
  }

  async RemoveReport (lReportId, connection) {
    const body = await this.baseRequest({
      command: `${className}.RemoveReport`,
      data: { lReportId },
      connection
    })
    return this.deserialize(body)
  }

  async ExecuteReportAsync ({ lReportId, pOptions }, connection) {
    const body = await this.baseRequest({
      command: `${className}.ExecuteReportAsync`,
      data: {
        lReportId,
        pOptions: this.serialize(pOptions)
      },
      connection
    })
    return this.getResult(body, 'strRequestId')
  }

  async ExecuteReportAsyncGetData (strRequestId, connection) {
    const body = await this.baseRequest({
      command: `${className}.ExecuteReportAsyncGetData`,
      data: {
        strRequestId,
        nChunkSize: 16336
      },
      connection
    })
    return this.getBody(body)
  }

  async ExecuteReportAsyncGetParsedData (strRequestId, connection) {
    const data = await this.ExecuteReportAsyncGetData(strRequestId, connection)

    return {
      result: {
        ...data.result,
        pXmlData: data.result.pXmlData === null
          ? null
          : Buffer.from(data.result.pXmlData, 'base64').toString('utf-8')
      }
    }
  }

  async GetDefaultReportInfo ({ lReportType }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetDefaultReportInfo`,
      data: {
        lReportType: parseInt(lReportType)
      },
      connection
    })
    return this.getResult(body, 'PxgRetVal')
  }

  async RequestStatisticsData (pRequestParams, connection) {
    const body = await this.baseRequest({
      command: `${className}.RequestStatisticsData`,
      data: {
        pRequestParams: this.serialize(pRequestParams)
      },
      connection
    })
    return this.getResult(body, 'strRequestId')
  }

  async GetStatisticsData (strRequestId, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetStatisticsData`,
      data: { strRequestId },
      connection
    })
    return this.deserialize(body, 'pResultData')
  }

  async ExecuteReportAsyncCancel ({strRequestId}, connection) {
    const body = await this.baseRequest({
      command: `${className}.ExecuteReportAsyncCancel`,
      data: {
        strRequestId
      },
      connection
    })
    return this.getBody(body)
  }

  async GetConstantOutputForReportType ({ lReportType, lXmlTargetType }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetConstantOutputForReportType`,
      data: {
        lReportType,
        lXmlTargetType
      },
      connection
    })
    return this.getBody(body)
  }

  async CreateChartPNG ({ pChartData, pOptions = { RPT_CHART_WIDTH: 300, RPT_CHART_HEIGHT: 300 } }, connection) {
    const body = await this.baseRequest({
      command: `${className}.CreateChartPNG`,
      data: {
        pChartData: this.serialize(pChartData),
        pOptions
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Force reset of statistics data
   * Force resets statistics data, for example, resets the status of "Virus attack" or
   * "Failed to perform the administration server task" after acquaintance with the detailed information
   * @param pRequestParams - params with a statuses that need to be "reset"
   */
  async ResetStatisticsData ({ pRequestParams }, connection) {
    const body = await this.baseRequest({
      command: `${className}.ResetStatisticsData`,
      data: {
        pRequestParams: this.serialize(pRequestParams)
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Enumerate ids of existing reports
   * Returns array of existing report ids
   * returns {Array<int>} array of event ids
   */
  async GetReportIds (noParams, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetReportIds`,
      data: {},
      connection
    })
    return this.getBody(body)
  }

  /**
   * Obtain report common data
   * Returns common data for specified report
   * @param lReportId - id of report
   * @returns {object} - report common data
   */
  async GetReportCommonData ({ lReportId }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetReportCommonData`,
      data: {
        lReportId
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Get filter settings
   * Gets filter settings
   * @param lReportType - report type
   * @returns {object} filter settings
   */
  async GetFilterSettings ({ lReportType }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetFilterSettings`,
      data: {
        lReportType
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Cancel waiting for report data from slave servers
   * @param strRequestId - identity of asynchronous operation
   */
  async ExecuteReportAsyncCancelWaitingForSlaves ({ strRequestId }, connection) {
    const body = await this.baseRequest({
      command: `${className}.ExecuteReportAsyncCancelWaitingForSlaves`,
      data: {
        strRequestId
      },
      connection
    })
    return this.getBody(body)
  }

  /**
   * Cancel ReportManager.RequestStatisticsData operation
   * @param strRequestId
   * @param connection
   * @returns {Promise<*>}
   * @constructor
   */
  async CancelStatisticsRequest ({ strRequestId }, connection) {
    const body = await this.baseRequest({
      command: `${className}.CancelStatisticsRequest`,
      data: {
        strRequestId
      },
      connection
    })
    return this.getBody(body)
  }
}
