const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'ServerTransportSettings'

module.exports = class ServerTransportSettings extends BaseOpenAPIClient {
  /**
   * Checks if feature is activated and certificate can be changed to some custom value
   * @param {string} szwCertType - certificate type
   */
  IsFeatureActive ({ szwCertType }, connection) {
    return this.baseRequest({
      command: `${className}.IsFeatureActive`,
      data: { szwCertType },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Returns current connection settings
   * @param {string} szwCertType - certificate type
   */
  GetCurrentConnectionSettings ({ szwCertType }, connection) {
    return this.baseRequest({
      command: `${className}.GetCurrentConnectionSettings`,
      data: { szwCertType },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Sets custom certificate for one of SC Server's SSL listener
   * @param {string} szwCertType - certificate type
   * @param {object} pCertData - params with custom certificate data
   */
  SetCustomSrvCertificate ({ szwCertType, pCertData }, connection) {
    return this.baseRequest({
      command: `${className}.SetCustomSrvCertificate`,
      data: {
        szwCertType,
        pCertData
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Sets or recreates default certificate
   * @param {string} szwCertType - certificate type
   * @param {Number} CERT_CHANGE_TIMEOUT - optional, seconds, certificate change timeout for reserve certificates. It should be set for reserve certificate only.
   * @param {object} TRSP_SETTINGS_CERT_DATA - optional. Params with certificate settings to create new certificate.
   * @param {string} TRSP_SETTINGS_FQDN - is in TRSP_SETTINGS_CERT_DATA, optional, certificates's FQDN, if absent, then curent (Administration Server) host's FQDN will be used instead.
   * @param {object} TRSP_SETTINGS_CERT - optional. Params with certificate data.
   */
  SetOrCreateDefaultCertificate ({ szwCertType, pSettings }, connection) {
    return this.baseRequest({
      command: `${className}.SetOrCreateDefaultCertificate`,
      data: {
        szwCertType,
        pSettings: this.serialize(pSettings)
      },
      connection
    })
      .then(this.getResult)
  }
}
