const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'SsContents'

module.exports = class SsContents extends BaseOpenAPIClient {
  /**
   * Enumerate contents of settings storage
   * Retrieves list of sections.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {Connection} connection
   * @constructor
   */
  SS_GetNames ({ wstrID, wstrProduct, wstrVersion }, connection) {
    return this.baseRequest({ command: `${className}.SS_GetNames`, data: { wstrID, wstrProduct, wstrVersion }, connection })
      .then(this.getResult)
  }

  /**
   * Read data from settings storage
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {Connection} connection
   * @constructor
   */
  Ss_Read ({ wstrID, wstrProduct, wstrVersion, wstrSection, skipDeserialization }, connection) {
    return this.baseRequest({ command: `${className}.Ss_Read`, data: { wstrID, wstrProduct, wstrVersion, wstrSection }, connection })
      .then(skipDeserialization ? this.getBody : this.deserialize)
  }

  /**
   * Add new data to settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {} pNewData - new data to write
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Add ({ wstrID, wstrProduct, wstrVersion, wstrSection, pNewData }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_Add`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection, pNewData: this.serialize(pNewData) },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Update existing data in settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {*} pNewData - new data to write
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Update ({ wstrID, wstrProduct, wstrVersion, wstrSection, pNewData }, connection) {
    let data = this.serialize(pNewData)
    return this.baseRequest({
      command: `${className}.Ss_Update`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection, pNewData: data},
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Replace data in settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {*} pNewData - new data to write
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Replace ({ wstrID, wstrProduct, wstrVersion, wstrSection, pNewData, skipSerialization}, connection) {
    let _pNewData = skipSerialization ? pNewData : this.serialize(pNewData)
    return this.baseRequest({
      command: `${className}.Ss_Replace`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection, pNewData: _pNewData },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Clear and write data in settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {*} pNewData - new data to write
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Clear ({ wstrID, wstrProduct, wstrVersion, wstrSection, pNewData }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_Clear`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection, pNewData },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Delete data from settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {*} pData - data
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Delete ({ wstrID, wstrProduct, wstrVersion, wstrSection, pData }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_Delete`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection, pData },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Create section in settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_CreateSection ({ wstrID, wstrProduct, wstrVersion, wstrSection }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_CreateSection`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Delete section from settings storage. Changes are not saved until method Ss_Apply is called.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {string} wstrProduct - product name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>
   * @param {string} wstrVersion - version string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {string} wstrSection - section name string, non-empty string, not longer than 31 character, and cannot contain characters /\:*?"<>.
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_DeleteSection ({ wstrID, wstrProduct, wstrVersion, wstrSection }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_DeleteSection`,
      data: { wstrID, wstrProduct, wstrVersion, wstrSection },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Save changes.
   * @param {string} wstrID - identifier of opened SsContents
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  Ss_Apply ({ wstrID }, connection) {
    return this.baseRequest({
      command: `${className}.Ss_Apply`,
      data: { wstrID },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Closes opened SsContents and releases associated server resources. After calling this method wstrID is not longer valid
   * @inheritdoc Ss_Apply
   */
  Ss_Release ({ wstrID }, connection) {
    return this.baseRequest({ command: `${className}.Ss_Release`, data: { wstrID }, connection })
      .then(this.getBody)
  }
}
