const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'TrafficManager'

module.exports = class TrafficManager extends BaseOpenAPIClient {
  /**
   * Add traffic restriction.
   *
   * @param {Object} { pRestriction } Restriction definition, see Traffic restrictions for details and attributes meaning
   * @param {*} connection
   * @returns {int} new restriction ID
   */
  async AddRestriction ({ pRestriction }, connection) {
    return this.baseRequest({
      command: `${className}.AddRestriction`,
      data: { pRestriction },
      connection
    }).then(this.getBody)
  }

  /**
   * Remove restriction by ID
   *
   * @param {Number} { nRestrictionId } Restricition ID to delete
   * @param {*} connection
   * @returns {*}
   */
  async DeleteRestriction ({ nRestrictionId }, connection) {
    return this.baseRequest({
      command: `${className}.DeleteRestriction`,
      data: { nRestrictionId },
      connection
    }).then(this.getBody)
  }

  /**
   * Get all active restrictions list
   *
   * @param {*} args
   * @param {*} connection
   * @returns {Promise<Array[Object]>} {pRestrictions} array of restrictions
   */
  async GetRestrictions ({ args = null }, connection) {
    const body = await this.baseRequest({
      command: `${className}.GetRestrictions`,
      connection
    })
    return this.deserialize(body, 'pRestrictions')
  }

  /**
   * Modify existing traffic restriction settings.
   *
   * @param {Object} { pRestriction } Restriction definition, see Traffic restrictions for details and attributes meaning
   * @param {Number} { nRestrictionId } restriction to modify. If restriction with such id does not exist then new restriction will be added and this parameters will be ignored.
   * @param {*} connection
   * @returns {Promise<Number>} modified restriction id. If restriction did not exist before call then newly created restriction id.
   */
  async UpdateRestriction ({ pRestriction, nRestrictionId }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateRestriction`,
      data: { pRestriction, nRestrictionId },
      connection
    }).then(this.getBody)
  }
}
