const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'VapmControlApi'

module.exports = class VapmControlApi extends BaseOpenAPIClient {
  /**
   * Changes updates approval
   * @param { Array<Object> } pUpdates - Array of updates to be approved/declined,
   * @param { number } nApprovementState - virtual server parent group
   * @returns { Promise<void> }
   */
  ChangeApproval ({ pUpdates, nApprovementState }, connection) {
    return this.baseRequest({
      command: `${className}.ChangeApproval`,
      data: {
        pUpdates: this.serialize(pUpdates),
        nApprovementState
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Requests the set of EULA descriptors for the given set of updates
   * @param { Array<Object> } pUpdates - Array of updates to request EULA's for
   * @param { number } nLcid - Preferred LCID
   * @returns { Promise<TResult> }
   */
  GetEulasInfo ({ pUpdates, nLcid }, connection) {
    return this.baseRequest({
      command: `${className}.GetEulasInfo`,
      data: {
        pUpdates: this.serialize(pUpdates),
        nLcid
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Requests the set of EULA ids for the given set of updates
   * @param { Array<Object> } pUpdates - Array of updates to request EULA's for
   * @param { number } nLcid - Preferred LCID
   * @returns { Promise<TResult> }
   */
  GetEulasIdsForUpdates ({ pUpdates, nLcid }, connection) {
    return this.baseRequest({
      command: `${className}.GetEulasIdsForUpdates`,
      data: {
        pUpdates: this.serialize(pUpdates),
        nLcid
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Requests the set of EULA ids for the given set of updates
   * @param { Array<Object> } pUpdates - Array of updates to request EULA's for
   * @param { number } nLcid - Preferred LCID
   * @returns { Promise<TResult> }
   */
  GetEulaParams ({ nEulaId }, connection) {
    return this.baseRequest({
      command: `${className}.GetEulaParams`,
      data: {
        nEulaId
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * RAccepts given EULAs
   * @param { Array<number> } pEulaIDs - Array of EULA IDs to accept
   * @returns { Promise<TResult> }
   */
  AcceptEulas ({ pEulaIDs }, connection) {
    return this.baseRequest({
      command: `${className}.AcceptEulas`,
      data: {
        pEulaIDs
      },
      connection
    })
      .then(this.getBody)
  }
}
